#include "xbasic_types.h"
#include "xio.h"

#include "xintc_l.h"
#include "xuartlite_l.h"

// defined in isr.c
extern void uart_handler(void *callback);
extern int uart_interrupt_count;
extern char uart_character;

/*
 * setup_interrupts: Initialize the interrupt sources and handlers
 *
 * Should be called once when the system starts
 *
 * The main _interrupt_handler() function from Xilinx
 *
 * Saves and restores CPU context, etc.
 *
 * Sees which interrupts are pending, and for each it
 *     acknowledges the interrupt and
 *     calls a user-defined interrupt handler in Xintc_InterruptVectorTable
 *
 * Place interrupt service routines in isr.c to ensure they are placed in
 * the proper memory segment.
 */
void setup_interrupts()
{
  /*
   * Reset the interrupt controller peripheral
   */

  /* Disable the interrupt signal */
  XIntc_mMasterDisable(XPAR_INTC_SINGLE_BASEADDR);

  /* Disable all interrupt sources */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,0);

  /* Acknowledge all possible interrupt sources
     to make sure none are pending */
  XIntc_mAckIntr(XPAR_INTC_SINGLE_BASEADDR, 0xffffffff);

  /*
   * Install the UART interrupt handler
   */

  XIntc_InterruptVectorTable[XPAR_INTC_MYUART_INTERRUPT_INTR].Handler =
    uart_handler;

  /*
   * Enable interrupt sources
   */

  /* Enable CPU interrupts */
  microblaze_enable_interrupts();

  /* Enable interrupts from the interrupt controller */
  XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR);

  /* Tell the interrupt controller to accept interrupts from the UART */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_MYUART_INTERRUPT_MASK);
  
  /* Enable UART interrupt generation */
  XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);
}

int main()
{
  char c;
  int i;
  char buf[2];
  buf[1] = '\0';

  // Enable the instruction cache: makes the code run 6 times faster

  microblaze_enable_icache();

   print("Hello Lab3!\r\n");

   XIo_Out16(0x00A00200, 0x1111);
   XIo_Out16(0x00A00202, 0x2222);
   XIo_Out16(0x00A00204, 0x3333);
   XIo_Out16(0x00A00208, 0x4444);

   //   for(i=0;i<32; i+=2){
     

     putnum(XIo_In16(0x00A00200));
     print("\r\n");
     putnum(XIo_In16(0x00A00200));
     print("\r\n");
     putnum(XIo_In16(0x00A00200));
     print("\r\n");
     putnum(XIo_In16(0x00A00202));
     print("\r\n");
     putnum(XIo_In16(0x00A00204));
     print("\r\n");
     putnum(XIo_In16(0x00A00206));
     print("\r\n");
     //   }

   return 0;

  setup_interrupts();
  
  for (;;) {

    print("Character number ");
    putnum(uart_interrupt_count);
    print(" is ");
    buf[0] = uart_character;
    print(buf);
    print("\r\n");
    
    for (i = 0 ; i < 10000000 ; i++ )  ; //delay 
  }

  return 0;
}
